# load packages
library(ids)

# generate random, but unique strings with 4 bytes each to represent genes #"915892f3"
set.seed(1307)
simgenes <- random_id(28395, 4, use_openssl = FALSE) 

#' @title intsct.bnom.test
#' @description Perform binomial test of intersection of elements of two lists. H0: observed intersection is equal to the intersection of random sets of same sizes. H1: observed intersection greater than the expected.
#' @param degs An integer. The number of deferentially expressed genes.
#' @param original An integer. The number of genes in the gene universe.
#' @param observed An integer. The number of elements of the observed intersection between "degs" and "original".
#' @return A list of two elements: the expected number of elements of the intersection and summary of the binomial test, including a one-sided p-value.

intsct.bnom.test <- function(degs, original, observed){
  intsct <- replicate(10000, 
            length(intersect(sample(simgenes, degs, replace = FALSE), 
                             sample(simgenes, original, replace = FALSE))))
  
  z <- round(median(intsct), 0)
  
  pintersect <- (z/degs)
  
  result <- binom.test(x = observed,
             n = degs,
             p = pintersect,
             alternative = "greater",
             conf.level = 0.95)
  
  lst <- list("expected" = z, "result" = result)
  
  return(lst)
}

# Using the function. See Table 2. In order, there are the # of degs, # of genes in list and # of observed intersection.
konig_brass <- intsct.bnom.test(208, 406, 4)
zhou <- intsct.bnom.test(208, 264, 2)
yeung <- intsct.bnom.test(208, 262, 3)
hivdb <- intsct.bnom.test(208, 4667, 64)
rnacentral <- intsct.bnom.test(208, 7972, 3)
